//___________________________________________
//				Joshua Schapiro
//					rs232.c
//			  created: 08/27/2010
//			  updated:
//
//___________________________________________


#include "usart.h"

#define Rs232_BufferSize		100

const char HexChars[] = "0123456789ABCDEF"; // fr rprintfFloat(char numDigits, double x);
#define hexchar(x)	pgm_read_byte( HexChars+((x)&0x0f) )


volatile uint8_t    Rs232Buffer[Rs232_BufferSize];
volatile uint16_t  	Rs232_readLocation = 0;
volatile uint16_t   Rs232_writeLocation = 0;
volatile bool		okToSendAirQuality = false;

char				airQualityString[100];
uint32_t			airCount[6];

void Rs232_Init(void){
	
	PORTE.DIRSET = PIN3_bm;  // Pin39 von PORTE3 (TXD0) ist Ausgang                          
	PORTE.DIRCLR = PIN2_bm;  // Pin38 von PORTE2 (RXD0) ist Eingang                          
   
	USARTE0.CTRLC = USART_CHSIZE_8BIT_gc | USART_PMODE_DISABLED_gc | (false ? USART_SBMODE_bm : 0);
	
	
	// Init USART 921600bps -> (F_CPU 48000000 = BAUDCTRLA = 0x21 / BAUDCTRLB = 0x91)	// nur AU Type HTerm 0.8.1  
	// Init USART 115200bps -> (F_CPU 48000000 = BAUDCTRLA = 0x85 / BAUDCTRLB = 0x9C)	// nur AU Type   
	// Init USART 115200bps -> (F_CPU 32000000 = BAUDCTRLA = 0x17 / BAUDCTRLB = 0xA4)	    				
	// Init USART 115200bps -> (F_CPU 16000000 = BAUDCTRLA = 0xD7 / BAUDCTRLB = 0x93)     
    // Init USART 115200bps -> (F_CPU 8000000  = BAUDCTRLA = 0xAC / BAUDCTRLB = 0x91)
	
	// Init USART 9600 bps  -> (F_CPU 32000000 = BAUDCTRLA = 0x7B / BAUDCTRLB = 0xD6)
	// Init USART 9600 bps  -> (F_CPU 16000000 = BAUDCTRLA = 0xE5 / BAUDCTRLB = 0xBC)

    //USARTF0.BAUDCTRLA = 0xE; 	// Init USART 576600bps F_CPU 32000000 
    //USARTF0.BAUDCTRLB = 0xA9;  // 0,00 % Abweichung 


    USARTE0.BAUDCTRLA = 0x17; 	// Init USART 115200bps F_CPU 32000000 
    USARTE0.BAUDCTRLB = 0xA4;   
 

	USARTE0.CTRLB |= USART_RXEN_bm;
	USARTE0.CTRLB |= USART_TXEN_bm;
	
	USARTE0.CTRLA |= USART_RXCINTLVL_HI_gc;
}



bool Rs232_CharReadyToRead(void){
	if(Rs232_writeLocation == Rs232_readLocation){
		return false;  
	} else { 
		return true;
	}
}

uint8_t Rs232_GetByte(bool blocking){
	if(blocking){
		while(!Rs232_CharReadyToRead());
	}

	uint8_t tmp = Rs232Buffer[Rs232_readLocation];
	Rs232_readLocation++;
	if(Rs232_readLocation >= Rs232_BufferSize){
	  Rs232_readLocation=0;
	}
	return tmp;
}

void Rs232_ClearBuffer(void){
	Rs232_readLocation = Rs232_writeLocation;
}

void Rs232_SendByte(uint8_t data){
	while(!(USARTE0.STATUS & USART_DREIF_bm));
	USARTE0.DATA = data;	
}

void Rs232_SendString(char string [],bool CR){
	for(uint8_t i = 0; i < strlen(string); i++){
		Rs232_SendByte(string[i]);
	}

  if(CR){
		Rs232_SendByte(13);
		Rs232_SendByte(10);
	}
}

// *** rprintfNum ***
// special printf for numbers only
// see formatting information below
//	Print the number "n" in the given "base"
//	using exactly "numDigits"
//	print +/- if signed flag "isSigned" is TRUE
//	use the character specified in "padchar" to pad extra characters
//
//	Examples:
//	uartPrintfNum(10, 6,  TRUE, ' ',   1234);  -->  " +1234"
//	uartPrintfNum(10, 6, FALSE, '0',   1234);  -->  "001234"
//	uartPrintfNum(16, 6, FALSE, '.', 0x5AA5);  -->  "..5AA5"

void rprintfNum(char base, char numDigits, char isSigned, char padchar, long n)
{
	// define a global HexChars or use line below
	//static char HexChars[16] = "0123456789ABCDEF";
	char *p, buf[32];
	unsigned long x;
	unsigned char count;

	// prepare negative number
	if( isSigned && (n < 0) )
	{
		x = -n;
	}
	else
	{
	 	x = n;
	}

	// setup little string buffer
	count = (numDigits-1)-(isSigned?1:0);
  	p = buf + sizeof (buf);
  	*--p = '\0';
	
	// force calculation of first digit
	// (to prevent zero from not printing at all!!!)
	*--p = hexchar(x%base); x /= base;
	// calculate remaining digits
	while(count--)
	{
		if(x != 0)
		{
			// calculate next digit
			*--p = hexchar(x%base); x /= base;
		}
		else
		{
			// no more digits left, pad out to desired length
			*--p = padchar;
		}
	}

	// apply signed notation if requested
	if( isSigned )
	{
		if(n < 0)
		{
   			*--p = '-';
		}
		else if(n > 0)
		{
	   		*--p = '+';
		}
		else
		{
	   		*--p = ' ';
		}
	}

	// print the string right-justified
	count = numDigits;
	while(count--)
	{
		Rs232_SendByte(*p++);
	}
}


ISR(USARTD0_RXC_vect){
	Rs232Buffer[Rs232_writeLocation] = USARTE0.DATA;

	if(Rs232Buffer[Rs232_writeLocation] == 0x0A){
		okToSendAirQuality = true;
	}
	Rs232_writeLocation++;
	if(Rs232_writeLocation >= Rs232_BufferSize){
		Rs232_writeLocation = 0;
	}
}
